@@ -22,13 +22,33 @@
 #include "plugin.h"
 #include "syncjournaldb.h"
 
+#include <QCoreApplication>
 #include <QDir>
-#include <QPluginLoader>
 #include <QLoggingCategory>
+#include <QPluginLoader>
 
+#ifdef Q_OS_WIN
+#include <qt_windows.h>
+#endif
 using namespace OCC;
 
-Q_LOGGING_CATEGORY(lcPlugin, "plugins", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcVfs, "sync.vfs", QtInfoMsg)
+
+namespace {
+QString modeToPluginName(Vfs::Mode mode)
+{
+    switch (mode) {
+    case Vfs::Off:
+        return QStringLiteral("off");
+    case Vfs::WithSuffix:
+        return QStringLiteral("suffix");
+    case Vfs::WindowsCfApi:
+        return QStringLiteral("win");
+    default:
+        Q_UNREACHABLE();
+    }
+}
+}
 
 Vfs::Vfs(QObject* parent)
     : QObject(parent)
@@ -42,20 +62,6 @@ QString Vfs::underlyingFileName(const QString &fileName) const
 
 Vfs::~Vfs() = default;
 
-QString Vfs::modeToString(Mode mode)
-{
-    // Note: Strings are used for config and must be stable
-    switch (mode) {
-    case Off:
-        return QStringLiteral("off");
-    case WithSuffix:
-        return QStringLiteral("suffix");
-    case WindowsCfApi:
-        return QStringLiteral("wincfapi");
-    }
-    return QStringLiteral("off");
-}
-
 Optional<Vfs::Mode> Vfs::modeFromString(const QString &str)
 {
     // Note: Strings are used for config and must be stable
@@ -66,7 +72,6 @@ Optional<Vfs::Mode> Vfs::modeFromString(const QString &str)
     } else if (str == QLatin1String("wincfapi")) {
         return WindowsCfApi;
     }
-    qCWarning(lcPlugin) << "Unknown VFS mode string: " << str;
     return Off;
 }
 
@@ -140,60 +145,93 @@ Vfs::AvailabilityResult Vfs::availabilityInDb(const QString &folderPath)
     return AvailabilityError::NoSuchItem;
 }
 
-static QString modeToPluginName(Vfs::Mode mode)
+void Vfs::wipeDehydratedVirtualFiles()
 {
-    switch (mode) {
-    case Vfs::Off:
-        return QStringLiteral("off");
-    case Vfs::WithSuffix:
-        return QStringLiteral("suffix");
-    case Vfs::WindowsCfApi:
-        return QStringLiteral("win");
-    default:
-        Q_UNREACHABLE();
+    if (mode() == Vfs::Mode::Off) {
+        // there are no placeholders
+        return;
     }
+    _setupParams->journal->getFilesBelowPath(QByteArray(), [&](const SyncJournalFileRecord &rec) {
+        // only handle dehydrated files
+        if (rec._type != ItemTypeVirtualFile && rec._type != ItemTypeVirtualFileDownload) {
+            return;
+        }
+        const QString relativePath = QString::fromUtf8(rec._path);
+        qCDebug(lcVfs) << "Removing db record for dehydrated file" << relativePath;
+        _setupParams->journal->deleteFileRecord(relativePath);
+
+        // If the local file is a dehydrated placeholder, wipe it too.
+        // Otherwise leave it to allow the next sync to have a new-new conflict.
+        const QString absolutePath = _setupParams->filesystemPath + relativePath;
+        if (QFile::exists(absolutePath)) {
+            // according to our db this is a dehydrated file, check it  to be sure
+            if (isDehydratedPlaceholder(absolutePath)) {
+                qCDebug(lcVfs) << "Removing local dehydrated placeholder" << relativePath;
+                FileSystem::remove(absolutePath);
+            }
+        }
+    });
+
+    _setupParams->journal->forceRemoteDiscoveryNextSync();
+
+    // Postcondition: No ItemTypeVirtualFile / ItemTypeVirtualFileDownload left in the db.
+    // But hydrated placeholders may still be around.
 }
 
-bool OCC::isVfsPluginAvailable(Vfs::Mode mode)
+Q_LOGGING_CATEGORY(lcPlugin, "sync.plugins", QtInfoMsg)
+
+OCC::VfsPluginManager *OCC::VfsPluginManager::_instance = nullptr;
+
+bool OCC::VfsPluginManager::isVfsPluginAvailable(Vfs::Mode mode) const
 {
-    // TODO: cache plugins available?
-    auto name = modeToPluginName(mode);
-    if (name.isEmpty())
-        return false;
-    auto pluginPath = pluginFileName(QStringLiteral("vfs"), name);
-    QPluginLoader loader(pluginPath);
+    {
+        auto result = _pluginCache.constFind(mode);
+        if (result != _pluginCache.cend()) {
+            return *result;
+        }
+    }
+    const bool out = [mode] {
+        const QString name = modeToPluginName(mode);
+        if (!OC_ENSURE_NOT(name.isEmpty())) {
+            return false;
+        }
+        auto pluginPath = pluginFileName(QStringLiteral("vfs"), name);
+        QPluginLoader loader(pluginPath);
 
-    auto basemeta = loader.metaData();
-    if (basemeta.isEmpty() || !basemeta.contains(QStringLiteral("IID"))) {
-        qCDebug(lcPlugin) << "Plugin doesn't exist" << loader.fileName();
-        return false;
-    }
-    if (basemeta[QStringLiteral("IID")].toString() != QLatin1String("org.owncloud.PluginFactory")) {
-        qCWarning(lcPlugin) << "Plugin has wrong IID" << loader.fileName() << basemeta[QStringLiteral("IID")];
-        return false;
-    }
+        auto basemeta = loader.metaData();
+        if (basemeta.isEmpty() || !basemeta.contains(QStringLiteral("IID"))) {
+            qCDebug(lcPlugin) << "Plugin doesn't exist:" << loader.fileName() << "LibraryPath:" << QCoreApplication::libraryPaths();
+            return false;
+        }
+        if (basemeta[QStringLiteral("IID")].toString() != QLatin1String("org.owncloud.PluginFactory")) {
+            qCWarning(lcPlugin) << "Plugin has wrong IID" << loader.fileName() << basemeta[QStringLiteral("IID")];
+            return false;
+        }
 
-    auto metadata = basemeta[QStringLiteral("MetaData")].toObject();
-    if (metadata[QStringLiteral("type")].toString() != QLatin1String("vfs")) {
-        qCWarning(lcPlugin) << "Plugin has wrong type" << loader.fileName() << metadata[QStringLiteral("type")];
-        return false;
-    }
-    if (metadata[QStringLiteral("version")].toString() != OCC::Version::version().toString()) {
-        qCWarning(lcPlugin) << "Plugin has wrong version" << loader.fileName() << metadata[QStringLiteral("version")];
-        return false;
-    }
+        auto metadata = basemeta[QStringLiteral("MetaData")].toObject();
+        if (metadata[QStringLiteral("type")].toString() != QLatin1String("vfs")) {
+            qCWarning(lcPlugin) << "Plugin has wrong type" << loader.fileName() << metadata[QStringLiteral("type")];
+            return false;
+        }
+        if (metadata[QStringLiteral("version")].toString() != OCC::Version::version().toString()) {
+            qCWarning(lcPlugin) << "Plugin has wrong version" << loader.fileName() << metadata[QStringLiteral("version")];
+            return false;
+        }
 
-    // Attempting to load the plugin is essential as it could have dependencies that
-    // can't be resolved and thus not be available after all.
-    if (!loader.load()) {
-        qCWarning(lcPlugin) << "Plugin failed to load:" << loader.errorString();
-        return false;
-    }
+        // Attempting to load the plugin is essential as it could have dependencies that
+        // can't be resolved and thus not be available after all.
+        if (!loader.load()) {
+            qCWarning(lcPlugin) << "Plugin failed to load:" << loader.errorString();
+            return false;
+        }
 
-    return true;
+        return true;
+    }();
+    _pluginCache[mode] = out;
+    return out;
 }
 
-Vfs::Mode OCC::bestAvailableVfsMode()
+Vfs::Mode OCC::VfsPluginManager::bestAvailableVfsMode() const
 {
     if (isVfsPluginAvailable(Vfs::WindowsCfApi)) {
         return Vfs::WindowsCfApi;
@@ -205,7 +243,7 @@ Vfs::Mode OCC::bestAvailableVfsMode()
     Q_UNREACHABLE();
 }
 
-std::unique_ptr<Vfs> OCC::createVfsFromPlugin(Vfs::Mode mode)
+std::unique_ptr<Vfs> OCC::VfsPluginManager::createVfsFromPlugin(Vfs::Mode mode) const
 {
     auto name = modeToPluginName(mode);
     if (name.isEmpty())
@@ -239,3 +277,39 @@ std::unique_ptr<Vfs> OCC::createVfsFromPlugin(Vfs::Mode mode)
     qCInfo(lcPlugin) << "Created VFS instance from plugin" << pluginPath;
     return vfs;
 }
+
+const VfsPluginManager &VfsPluginManager::instance()
+{
+    if (!_instance) {
+        _instance = new VfsPluginManager();
+    }
+    return *_instance;
+}
+
+template <>
+QString Utility::enumToString(Vfs::Mode mode)
+{
+    // Note: Strings are used for config and must be stable
+    switch (mode) {
+    case Vfs::Mode::WithSuffix:
+        return QStringLiteral("suffix");
+    case Vfs::Mode::WindowsCfApi:
+        return QStringLiteral("wincfapi");
+    case Vfs::Mode::Off:
+        return QStringLiteral("off");
+    }
+    Q_UNREACHABLE();
+}
+
+VfsSetupParams::VfsSetupParams(const AccountPtr &account, const QUrl &baseUrl, bool groupInSidebar, SyncEngine *syncEngine)
+    : account(account)
+    , _baseUrl(baseUrl)
+    , _groupInSidebar(groupInSidebar)
+    , _syncEngine(syncEngine)
+{
+}
+
+SyncEngine *VfsSetupParams::syncEngine() const
+{
+    return _syncEngine;
+}
