@@ -15,17 +15,16 @@
  */
 
 #include "socketapi.h"
+#include "scheduling/syncscheduler.h"
 #include "socketapi_p.h"
 
-#include "account.h"
+#include "gui/commonstrings.h"
+
 #include "accountmanager.h"
-#include "accountstate.h"
-#include "capabilities.h"
 #include "common/asserts.h"
+#include "common/depreaction.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/version.h"
-#include "config.h"
-#include "configfile.h"
 #include "filesystem.h"
 #include "folder.h"
 #include "folderman.h"
@@ -141,22 +140,21 @@ SocketApi::SocketApi(QObject *parent)
     qRegisterMetaType<QSharedPointer<SocketApiJob>>("QSharedPointer<SocketApiJob>");
     qRegisterMetaType<QSharedPointer<SocketApiJobV2>>("QSharedPointer<SocketApiJobV2>");
 
-    const QString socketPath = Utility::socketApiSocketPath();
+    _socketPath = Utility::socketApiSocketPath();
 
     // Remove any old socket that might be lying around:
-    SocketApiServer::removeServer(socketPath);
+    SocketApiServer::removeServer(_socketPath);
 
     // Create the socket path:
     if (!Utility::isMac()) {
         // Not on macOS: there the directory is there, and created for us by the sandboxing
         // environment, because we belong to an App Group.
-        QFileInfo info(socketPath);
+        QFileInfo info(_socketPath);
         if (!info.dir().exists()) {
             bool result = info.dir().mkpath(QStringLiteral("."));
             qCDebug(lcSocketApi) << "creating" << info.dir().path() << result;
             if (result) {
-                QFile::setPermissions(socketPath,
-                    QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
+                QFile::setPermissions(_socketPath, QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
             }
         }
     }
@@ -164,18 +162,11 @@ SocketApi::SocketApi(QObject *parent)
     // Wire up the server instance to us, so we can accept new connections:
     connect(&_localServer, &SocketApiServer::newConnection, this, &SocketApi::slotNewConnection);
 
-    // Start listeneing:
-    if (_localServer.listen(socketPath)) {
-        qCInfo(lcSocketApi) << "server started, listening at " << socketPath;
-    } else {
-        qCWarning(lcSocketApi) << "can't start server" << socketPath;
-    }
-
-    // folder watcher
-    connect(FolderMan::instance(), &FolderMan::folderSyncStateChange, this, &SocketApi::slotUpdateFolderView);
-
-    // Now we're ready to start the native shell integration:
-    Utility::startShellIntegration();
+    connect(AccountManager::instance(), &AccountManager::accountRemoved, this, [this](const auto &accountState) {
+        if (_registeredAccounts.contains(accountState->account())) {
+            unregisterAccount(accountState->account());
+        }
+    });
 }
 
 SocketApi::~SocketApi()
@@ -187,6 +178,19 @@ SocketApi::~SocketApi()
     _listeners.clear();
 }
 
+void SocketApi::startShellIntegration()
+{
+    // Start listeneing:
+    if (_localServer.listen(_socketPath)) {
+        qCInfo(lcSocketApi) << "server started, listening at " << _socketPath;
+    } else {
+        qCWarning(lcSocketApi) << "can't start server" << _socketPath;
+    }
+
+    // Now we're ready to start the native shell integration:
+    Utility::startShellIntegration();
+}
+
 void SocketApi::slotNewConnection()
 {
     // Note that on macOS this is not actually a line-based QIODevice, it's a SocketApiSocket which is our
@@ -256,7 +260,7 @@ void SocketApi::slotReadSocket()
             } else {
                 functionWithArguments += command.toUtf8() + QByteArrayLiteral("(QString,SocketListener*)");
             }
-            Q_ASSERT(staticQtMetaObject.normalizedSignature(functionWithArguments.constData()) == functionWithArguments);
+            Q_ASSERT(staticMetaObject.normalizedSignature(functionWithArguments.constData()) == functionWithArguments);
             const auto out = staticMetaObject.indexOfMethod(functionWithArguments.constData());
             if (out == -1) {
                 listener->sendError(QStringLiteral("Function %1 not found").arg(QString::fromUtf8(functionWithArguments)));
@@ -265,7 +269,7 @@ void SocketApi::slotReadSocket()
             return out;
         }();
 
-        const auto argument = argPos != -1 ? line.midRef(argPos + 1) : QStringRef();
+        const auto argument = argPos != -1 ? line.mid(argPos + 1) : QString();
         if (command.startsWith(QLatin1String("ASYNC_"))) {
             auto arguments = argument.split(QLatin1Char('|'));
             if (arguments.size() != 2) {
@@ -277,8 +281,7 @@ void SocketApi::slotReadSocket()
 
             auto jobId = arguments[0];
 
-            auto socketApiJob = QSharedPointer<SocketApiJob>(
-                new SocketApiJob(jobId.toString(), listener, json), &QObject::deleteLater);
+            auto socketApiJob = QSharedPointer<SocketApiJob>(new SocketApiJob(jobId, listener, json), &QObject::deleteLater);
             if (indexOfMethod != -1) {
                 staticMetaObject.method(indexOfMethod)
                     .invoke(this, Qt::QueuedConnection,
@@ -292,7 +295,7 @@ void SocketApi::slotReadSocket()
             QJsonParseError error;
             const auto json = QJsonDocument::fromJson(argument.toUtf8(), &error).object();
             if (error.error != QJsonParseError::NoError) {
-                qCWarning(lcSocketApi()) << "Invalid json" << argument.toString() << error.errorString();
+                qCWarning(lcSocketApi()) << "Invalid json" << argument << error.errorString();
                 listener->sendError(error.errorString());
                 return;
             }
@@ -310,9 +313,7 @@ void SocketApi::slotReadSocket()
             if (indexOfMethod != -1) {
                 // to ensure that listener is still valid we need to call it with Qt::DirectConnection
                 OC_ASSERT(thread() == QThread::currentThread())
-                staticMetaObject.method(indexOfMethod)
-                    .invoke(this, Qt::DirectConnection, Q_ARG(QString, argument.toString()),
-                        Q_ARG(SocketListener*, listener.data()));
+                staticMetaObject.method(indexOfMethod).invoke(this, Qt::DirectConnection, Q_ARG(QString, argument), Q_ARG(SocketListener *, listener.data()));
             }
         }
     }
@@ -396,6 +397,8 @@ void SocketApi::slotUpdateFolderView(Folder *f)
         case OCC::SyncResult::SyncRunning:
             Q_FALLTHROUGH();
         case OCC::SyncResult::SyncAbortRequested:
+            [[fallthrough]];
+        case OCC::SyncResult::Offline:
             qCDebug(lcSocketApi) << "Not sending UPDATE_VIEW for" << f->path() << "because status() is" << f->syncResult().status();
         }
     }
@@ -453,7 +456,7 @@ void SocketApi::broadcastStatusPushMessage(const QString &systemPath, SyncFileSt
 {
     QString msg = buildMessage(QStringLiteral("STATUS"), systemPath, fileStatus.toSocketAPIString());
     Q_ASSERT(!systemPath.endsWith(QLatin1Char('/')));
-    uint directoryHash = qHash(systemPath.left(systemPath.lastIndexOf(QLatin1Char('/'))));
+    auto directoryHash = qHash(systemPath.left(systemPath.lastIndexOf(QLatin1Char('/'))));
     for (const auto &listener : qAsConst(_listeners)) {
         listener->sendMessageIfDirectoryMonitored(msg, directoryHash);
     }
@@ -624,9 +627,11 @@ void SocketApi::fetchPrivateLinkUrlHelper(const QString &localFile, const std::f
         return;
     }
 
-    auto record = fileData.journalRecord();
-    if (!record.isValid())
-        return;
+    if (!fileData.isSyncFolder()) {
+        auto record = fileData.journalRecord();
+        if (!record.isValid())
+            return;
+    }
 
     fetchPrivateLinkUrl(
         fileData.folder->accountState()->account(),
@@ -653,14 +658,17 @@ void SocketApi::command_OPEN_PRIVATE_LINK(const QString &localFile, SocketListen
 
 void SocketApi::command_OPEN_PRIVATE_LINK_VERSIONS(const QString &localFile, SocketListener *)
 {
-    auto openVersionsLink = [](const QUrl &link) {
-        QUrl url(link);
-        QUrlQuery query(url);
-        query.addQueryItem(QStringLiteral("details"), QStringLiteral("versionsTabView"));
-        url.setQuery(query);
-        Utility::openBrowser(url, nullptr);
-    };
-    fetchPrivateLinkUrlHelper(localFile, openVersionsLink);
+    const auto fileData = FileData::get(localFile);
+    if (fileData.isValid() && fileData.folder->accountState()->account()->capabilities().filesSharing().sharing_roles) {
+        fetchPrivateLinkUrl(fileData.folder->accountState()->account(), fileData.folder->webDavUrl(), fileData.serverRelativePath, this, [](const QUrl &url) {
+            const auto queryUrl = Utility::concatUrlPath(url, QString(), {{QStringLiteral("details"), QStringLiteral("versions")}});
+            Utility::openBrowser(queryUrl, nullptr);
+        });
+    } else {
+        fetchPrivateLinkUrlHelper(localFile, [this](const QUrl &link) {
+            Utility::openBrowser(Utility::concatUrlPath(link, {}, {{QStringLiteral("details"), QStringLiteral("versionsTabView")}}), nullptr);
+        });
+    }
 }
 
 void SocketApi::copyUrlToClipboard(const QUrl &link)
@@ -678,11 +686,11 @@ void SocketApi::command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketLi
             continue;
 
         // Update the pin state on all items
-        data.folder->vfs().setPinState(data.folderRelativePath, PinState::AlwaysLocal);
+        std::ignore = data.folder->vfs().setPinState(data.folderRelativePath, PinState::AlwaysLocal);
 
         // Trigger sync
         data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
-        data.folder->scheduleThisFolderSoon();
+        FolderMan::instance()->scheduler()->enqueueFolder(data.folder, SyncScheduler::Priority::Medium);
     }
 }
 
@@ -697,11 +705,11 @@ void SocketApi::command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener
             continue;
 
         // Update the pin state on all items
-        data.folder->vfs().setPinState(data.folderRelativePath, PinState::OnlineOnly);
+        std::ignore = data.folder->vfs().setPinState(data.folderRelativePath, PinState::OnlineOnly);
 
         // Trigger sync
         data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
-        data.folder->scheduleThisFolderSoon();
+        FolderMan::instance()->scheduler()->enqueueFolder(data.folder, SyncScheduler::Priority::Medium);
     }
 }
 
@@ -770,7 +778,7 @@ void SocketApi::command_MOVE_ITEM(const QString &localFile, SocketListener *)
     }
 }
 
-Q_INVOKABLE void OCC::SocketApi::command_OPEN_APP_LINK(const QString &localFile, SocketListener *listener)
+Q_INVOKABLE void OCC::SocketApi::command_OPEN_APP_LINK(const QString &localFile, [[maybe_unused]] SocketListener *listener)
 {
     const auto data = FileData::get(localFile);
     if (OC_ENSURE(data.folder)) {
@@ -786,9 +794,10 @@ void SocketApi::command_V2_LIST_ACCOUNTS(const QSharedPointer<SocketApiJobV2> &j
 {
     QJsonArray out;
     for (auto acc : AccountManager::instance()->accounts()) {
-        out << QJsonObject({ { QStringLiteral("name"), acc->account()->displayName() },
-            { QStringLiteral("id"), acc->account()->id() },
-            { QStringLiteral("uuid"), acc->account()->uuid().toString(QUuid::WithoutBraces) } });
+        OC_DISABLE_DEPRECATED_WARNING; // allow use of id
+        out << QJsonObject({{QStringLiteral("name"), acc->account()->displayName()}, {QStringLiteral("id"), acc->account()->id()},
+            {QStringLiteral("uuid"), acc->account()->uuid().toString(QUuid::WithoutBraces)}});
+        OC_ENABLE_DEPRECATED_WARNING
     }
     job->success({ { QStringLiteral("accounts"), out } });
 }
@@ -898,7 +907,12 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
             && !capabilities.sharePublicLinkEnforcePasswordForReadOnly();
 
         if (canCreateDefaultPublicLink) {
-            listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Create and copy public link to clipboard"));
+            if (fileData.folder->accountState()->supportsSpaces()) {
+                // TODO: See https://github.com/owncloud/client/issues/10845 : oCIS is getting a new sharing API, waiting for that before implementing a
+                // temporary solution.
+            } else {
+                listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Create and copy public link to clipboard"));
+            }
         } else if (publicLinksEnabled) {
             listener->sendMessage(QStringLiteral("MENU_ITEM:MANAGE_PUBLIC_LINKS") + flagString + tr("Copy public link to clipboard"));
         }
@@ -962,6 +976,11 @@ SocketApi::FileData SocketApi::FileData::parentFolder() const
     return FileData::get(QFileInfo(localPath).dir().path());
 }
 
+bool SocketApi::FileData::isValid() const
+{
+    return folder;
+}
+
 void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListener *listener)
 {
     listener->sendMessage(QStringLiteral("GET_MENU_ITEMS:BEGIN"));
@@ -970,29 +989,28 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
     // Some options only show for single files
     if (files.size() == 1) {
         const FileData fileData = FileData::get(files.first());
-        if (!fileData.isSyncFolder()) {
-            const auto record = fileData.journalRecord();
-            const bool isOnTheServer = record.isValid();
-            const auto flagString = isOnTheServer ? QLatin1String("::") : QLatin1String(":d:");
+        if (fileData.isValid() && fileData.folder->accountState()->isConnected()) {
+            const auto &capabilities = fileData.folder->accountState()->account()->capabilities();
+            if (!fileData.isSyncFolder()) {
+                const auto record = fileData.journalRecord();
+                const bool isOnTheServer = record.isValid();
+                const auto flagString = isOnTheServer ? QLatin1String("::") : QLatin1String(":d:");
 
-            const auto app = fileData.folder->accountState()->account()->appProvider().app(fileData.localPath);
-            if (!app.defaultApplication.isEmpty()) {
-                listener->sendMessage(QStringLiteral("MENU_ITEM:OPEN_APP_LINK") + flagString + tr("Open in %1").arg(app.defaultApplication));
-            }
+                const auto app = fileData.folder->accountState()->account()->appProvider().app(fileData.localPath);
+                if (!app.defaultApplication.isEmpty()) {
+                    listener->sendMessage(QStringLiteral("MENU_ITEM:OPEN_APP_LINK") + flagString + tr("Open in %1").arg(app.defaultApplication));
+                }
 
-            if (fileData.folder->accountState()->isConnected()) {
                 sendSharingContextMenuOptions(fileData, listener);
 
-                const auto &capabilities = fileData.folder->accountState()->account()->capabilities();
                 if (capabilities.privateLinkPropertyAvailable()) {
-                    listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK") + flagString + tr("Open in browser"));
+                    listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK") + flagString + CommonStrings::showInWebBrowser());
                 }
                 // Add link to versions pane if possible
-                if (capabilities.versioningEnabled()
-                    && capabilities.privateLinkDetailsParamAvailable()
-                    && isOnTheServer
-                    && !record.isDirectory()) {
-                    listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK_VERSIONS") + flagString + tr("Show file versions in browser"));
+                if (capabilities.versioningEnabled() && isOnTheServer && !record.isDirectory()) {
+                    if (capabilities.privateLinkDetailsParamAvailable() || capabilities.filesSharing().sharing_roles) {
+                        listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK_VERSIONS") + flagString + tr("Show file versions in web browser"));
+                    }
                 }
 
                 // Conflict files get conflict resolution actions
@@ -1005,10 +1023,8 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
                     const bool canAddToDir = !parentRecord._remotePerm.isNull()
                         && ((fileInfo.isFile() && parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddFile))
                             || (fileInfo.isDir() && parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddSubDirectories)));
-                    const bool canChangeFile =
-                        !isOnTheServer
-                        || (record._remotePerm.hasPermission(RemotePermissions::CanDelete)
-                            && record._remotePerm.hasPermission(RemotePermissions::CanMove)
+                    const bool canChangeFile = !isOnTheServer
+                        || (record._remotePerm.hasPermission(RemotePermissions::CanDelete) && record._remotePerm.hasPermission(RemotePermissions::CanMove)
                             && record._remotePerm.hasPermission(RemotePermissions::CanRename));
 
                     if (isConflict && canChangeFile) {
@@ -1043,6 +1059,11 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
                         }
                     }
                 }
+            } else {
+                // we are a sync root
+                if (capabilities.privateLinkPropertyAvailable()) {
+                    listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK::") + CommonStrings::showInWebBrowser());
+                }
             }
         }
     }
@@ -1102,8 +1123,7 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
         // TODO: Should be a submenu, should use icons
         auto makePinContextMenu = [&](bool makeAvailableLocally, bool freeSpace) {
             listener->sendMessage(QStringLiteral("MENU_SEPARATOR:d::"));
-            listener->sendMessage(QStringLiteral("MENU_ITEM:CURRENT_PIN:d:")
-                + Utility::vfsCurrentAvailabilityText(*combined));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:CURRENT_PIN:d:") + Utility::enumToDisplayName(combined.get()));
             listener->sendMessage(QStringLiteral("MENU_ITEM:MAKE_AVAILABLE_LOCALLY:")
                 + (makeAvailableLocally ? QStringLiteral(":") : QStringLiteral("d:"))
                 + Utility::vfsPinActionText());
