@@ -225,7 +225,7 @@ void logCredentialsJobResult(CredentialJob *credentialsJob)
     qCDebug(lcOauth) << "credentials job has finished";
 
     if (!credentialsJob->data().isValid()) {
-        qCCritical(lcOauth) << "Failed to read client id" << credentialsJob->errorString();
+        qCInfo(lcOauth) << "Failed to read client id" << credentialsJob->errorString();
     }
 }
 }
@@ -249,9 +249,16 @@ void OAuth::startAuthentication()
     qCDebug(lcOauth) << "starting authentication";
 
     // Listen on the socket to get a port which will be used in the redirect_uri
-    if (!_server.listen(QHostAddress::LocalHost)) {
+
+    for (const auto port : Theme::instance()->oauthPorts()) {
+        if (_server.listen(QHostAddress::LocalHost, port)) {
+            break;
+        }
+        qCDebug(lcOauth) << "Creating local server Port:" << port << "failed. Error:" << _server.errorString();
+    }
+    if (!_server.isListening()) {
         qCDebug(lcOauth) << "server is not listening";
-        emit result(NotSupported, QString());
+        emit result(Error, {});
         return;
     }
 
@@ -343,8 +350,13 @@ void OAuth::startAuthentication()
                     const QString tokenType = getRequiredField(data, QStringLiteral("token_type"), &fieldsError).toString().toLower();
                     const QUrl messageUrl = QUrl::fromEncoded(data[QStringLiteral("message_url")].toByteArray());
 
+                    const auto validTokenType = tokenType == QStringLiteral("bearer") || tokenType == QStringLiteral("app-password");
+
+                    qCDebug(lcOauth) << "STACK: Received user_id:" << user;
+
                     if (reply->error() != QNetworkReply::NoError || jsonParseError.error != QJsonParseError::NoError
-                        || !fieldsError.isEmpty()) {
+                        || !fieldsError.isEmpty()
+                        || !validTokenType) {
                         // do we have error message suitable for users?
                         QString errorReason = data[QStringLiteral("error_description")].toString();
                         if (errorReason.isEmpty()) {
@@ -360,7 +372,7 @@ void OAuth::startAuthentication()
                         } else if (jsonParseError.error != QJsonParseError::NoError) {
                             errorReason = tr("Could not parse the JSON returned from the server: <br><em>%1</em>")
                                               .arg(jsonParseError.errorString());
-                        } else if (tokenType != QStringLiteral("bearer")) {
+                        } else if (!validTokenType) {
                             errorReason = tr("Unsupported token type: %1").arg(tokenType);
                         } else if (!fieldsError.isEmpty()) {
                             errorReason = tr("The reply from the server did not contain all expected fields\n:%1").arg(fieldsError);
@@ -372,81 +384,83 @@ void OAuth::startAuthentication()
                             tr("Login Error"), tr("<h1>Login Error</h1><p>%1</p>").arg(errorReason));
                         emit result(Error);
                         return;
-                    } else if (tokenType == QLatin1String("app-password")) {
-                        finalize(socket, accessToken, refreshToken, user, user, messageUrl);
-                        return;
-                    } else if (tokenType != QLatin1String("bearer")) {
-                         QString errorReason = tr("Unsupported token type: %1").arg(tokenType);
-                         httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"),
-                             tr("<h1>Login Error</h1><p>%1</p>").arg(errorReason));
-                         emit result(Error);
-                         return;
                     }
 
-                    auto *job = FetchUserInfoJobFactory::fromOAuth2Credentials(_networkAccessManager, accessToken).startJob(_serverUrl, this);
+                    // We need to set the username here, so we can set it when we convert the authType back to basicAuth:
+                    setDavUser(user);
+                    qCDebug(lcOauth) << "STACK: Set _davUser:" << davUser();
+                    finalize(socket, accessToken, refreshToken, messageUrl);
+                    return; // Skip rest of this function
 
-                    connect(job, &CoreJob::finished, this, [=]() {
-                        if (!job->success()) {
-                            httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"),
-                                tr("Login Error"), tr("<h1>Login Error</h1><p>%1</p>").arg(job->errorMessage()));
-                            emit result(Error);
-                        } else {
-                            auto result = job->result().value<FetchUserInfoResult>();
-                            finalize(socket, accessToken, refreshToken, result.userName(), result.displayName(), messageUrl);
-                        }
-                    });
+                    if (!_davUser.isEmpty()) {
+                        auto *job = FetchUserInfoJobFactory::fromOAuth2Credentials(_networkAccessManager, accessToken).startJob(_serverUrl, this);
+
+                        connect(job, &CoreJob::finished, this, [=]() {
+                            if (!job->success()) {
+                                httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"), tr("Login Error"),
+                                    tr("<h1>Login Error</h1><p>%1</p>").arg(job->errorMessage()));
+                                emit result(Error);
+                            } else {
+                                auto fetchUserInfo = job->result().value<FetchUserInfoResult>();
+
+                                // note: the username still shouldn't be empty
+                                Q_ASSERT(!_davUser.isEmpty());
+
+                                // dav usernames are case-insensitive, we might compare a user input with the string provided by the server
+                                if (fetchUserInfo.userName().compare(_davUser, Qt::CaseInsensitive) != 0) {
+                                    // Connected with the wrong user
+                                    qCWarning(lcOauth) << "We expected the user" << _davUser << "but the server answered with user" << fetchUserInfo.userName();
+                                    const QString message = tr("<h1>Wrong user</h1>"
+                                                               "<p>You logged-in with user <em>%1</em>, but must login with user <em>%2</em>.<br>"
+                                                               "Please return to the %3 client and restart the authentication.</p>")
+                                                                .arg(fetchUserInfo.userName(), _davUser, Theme::instance()->appNameGUI());
+                                    httpReplyAndClose(socket, QStringLiteral("403 Forbidden"), tr("Wrong user"), message);
+                                    emit result(Error);
+                                } else {
+                                    finalize(socket, accessToken, refreshToken, messageUrl);
+                                }
+                            }
+                        });
+                    }
                 });
             });
         }
     });
 }
 
-void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QString &userName, const QString &displayName, const QUrl &messageUrl)
+void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QUrl &messageUrl)
 {
-    // dav user names are case insensetive, we might compare a user input with the string provided by the server
-    if (!_davUser.isEmpty() && userName.compare(_davUser, Qt::CaseInsensitive) != 0) {
-        // Connected with the wrong user
-        qCWarning(lcOauth) << "We expected the user" << _davUser << "but the server answered with user" << userName;
-        const QString message = tr("<h1>Wrong user</h1>"
-                                   "<p>You logged-in with user <em>%1</em>, but must login with user <em>%2</em>.<br>"
-                                   "Please return to the %3 client and restart the authentication.</p>")
-                                    .arg(userName, _davUser, Theme::instance()->appNameGUI());
-        httpReplyAndClose(socket, QStringLiteral("403 Forbidden"), tr("Wrong user"), message);
-        emit result(Error);
-        return;
-    }
-    const QString loginSuccessfullHtml = tr("<h1>Login Successful</h1><p>You can close this window.</p>");
-    const QString loginSuccessfullTitle = tr("Login Successful");
+    const QString loginSuccessfulHtml = tr("<h1>Login Successful</h1><p>You can close this window.</p>");
+    const QString loginSuccessfulTitle = tr("Login Successful");
     if (messageUrl.isValid()) {
-        httpReplyAndClose(socket, QStringLiteral("303 See Other"), loginSuccessfullTitle, loginSuccessfullHtml,
-            { QStringLiteral("Location: %1").arg(QString::fromUtf8(messageUrl.toEncoded())) });
+        httpReplyAndClose(socket, QStringLiteral("303 See Other"), loginSuccessfulTitle, loginSuccessfulHtml,
+            {QStringLiteral("Location: %1").arg(QString::fromUtf8(messageUrl.toEncoded()))});
     } else {
-        httpReplyAndClose(socket, QStringLiteral("200 OK"), loginSuccessfullTitle, loginSuccessfullHtml);
-    }
-    if (refreshToken == QStringLiteral("")) {
-        emit result(LoggedInAppPassword, userName, accessToken, refreshToken);
-    } else {
-        emit result(LoggedIn, userName, accessToken, refreshToken);
+        httpReplyAndClose(socket, QStringLiteral("200 OK"), loginSuccessfulTitle, loginSuccessfulHtml);
     }
+    emit result(LoggedIn, accessToken, refreshToken);
 }
 
-QNetworkReply *OAuth::postTokenRequest(const QList<QPair<QString, QString>> &queryItems)
+QNetworkReply *OAuth::postTokenRequest(QUrlQuery &&queryItems)
 {
-    const QUrl requestTokenUrl = _tokenEndpoint.isEmpty() ? Utility::concatUrlPath(_serverUrl, QStringLiteral("/request-client-token/finalize")) : _tokenEndpoint;
+    const QUrl requestTokenUrl =  Utility::concatUrlPath(_serverUrl, QStringLiteral("/request-client-token/finalize"));
     QNetworkRequest req;
     req.setTransferTimeout(defaultTimeoutMs());
-    const QByteArray basicAuth = QStringLiteral("%1:%2").arg(_clientId, _clientSecret).toUtf8().toBase64();
-    req.setRawHeader("Authorization", "Basic " + basicAuth);
+    switch (_endpointAuthMethod) {
+    case TokenEndpointAuthMethods::client_secret_basic:
+        req.setRawHeader("Authorization", "Basic " + QStringLiteral("%1:%2").arg(_clientId, _clientSecret).toUtf8().toBase64());
+        break;
+    case TokenEndpointAuthMethods::client_secret_post:
+        queryItems.addQueryItem(QStringLiteral("client_id"), _clientId);
+        queryItems.addQueryItem(QStringLiteral("client_secret"), _clientSecret);
+        break;
+    }
     req.setHeader(QNetworkRequest::ContentTypeHeader, QStringLiteral("application/x-www-form-urlencoded; charset=UTF-8"));
     req.setAttribute(HttpCredentials::DontAddCredentialsAttribute, true);
 
-    QUrlQuery arguments;
-    arguments.setQueryItems(QList<QPair<QString, QString>> { { QStringLiteral("client_id"), _clientId },
-                                { QStringLiteral("client_secret"), _clientSecret },
-                                { QStringLiteral("scope"), Theme::instance()->openIdConnectScopes() } }
-        << queryItems);
+    queryItems.addQueryItem(QStringLiteral("scope"), Theme::instance()->openIdConnectScopes());
     req.setUrl(requestTokenUrl);
-    return _networkAccessManager->post(req, arguments.toString(QUrl::FullyEncoded).toUtf8());
+    return _networkAccessManager->post(req, queryItems.toString(QUrl::FullyEncoded).toUtf8());
 }
 
 QByteArray OAuth::generateRandomString(size_t size) const
@@ -480,16 +494,24 @@ QUrl OAuth::authorisationLink() const
         // oc 10
         query.addQueryItem(QStringLiteral("user"), davUser);
     }
-    const QUrl url = _authEndpoint.isValid()
-        ? Utility::concatUrlPath(_authEndpoint, {}, query)
-        : Utility::concatUrlPath(_serverUrl, QStringLiteral("/request-client-token/authorize"), query);
+    const QUrl url = Utility::concatUrlPath(_serverUrl, QStringLiteral("/request-client-token/authorize"), query);
 
     return url;
 }
 
+void OAuth::setDavUser(const QString &davUser)
+{
+    _davUser = davUser;
+}
+
+QString OAuth::davUser() const
+{
+    return _davUser;
+}
+
 void OAuth::saveDynamicRegistrationDataForAccount(const OCC::AccountPtr &accountPtr, const QVariantMap &dynamicRegistrationData)
 {
-    auto credentialsJob = accountPtr->credentialManager()->set(dynamicRegistrationDataC(), dynamicRegistrationData);
+    accountPtr->credentialManager()->set(dynamicRegistrationDataC(), dynamicRegistrationData);
 }
 
 void OAuth::fetchWellKnown()
@@ -530,6 +552,16 @@ void OAuth::fetchWellKnown()
                 _tokenEndpoint = QUrl::fromEncoded(data[QStringLiteral("token_endpoint")].toString().toUtf8());
                 _registrationEndpoint = QUrl::fromEncoded(data[QStringLiteral("registration_endpoint")].toString().toUtf8());
                 _redirectUrl = QStringLiteral("http://127.0.0.1");
+
+                const auto authMethods = data.value(QStringLiteral("token_endpoint_auth_methods_supported")).toArray();
+                if (authMethods.contains(QStringLiteral("client_secret_basic"))) {
+                    _endpointAuthMethod = TokenEndpointAuthMethods::client_secret_basic;
+                } else if (authMethods.contains(QStringLiteral("client_secret_post"))) {
+                    _endpointAuthMethod = TokenEndpointAuthMethods::client_secret_post;
+                } else {
+                    OC_ASSERT_X(false, qPrintable(QStringLiteral("Unsupported token_endpoint_auth_methods_supported: %1").arg(QDebug::toString(authMethods))));
+                }
+
                 qCDebug(lcOauth) << "parsing .well-known reply successful, auth endpoint" << _authEndpoint
                                  << "and token endpoint" << _tokenEndpoint
                                  << "and registration endpoint" << _registrationEndpoint;
@@ -578,10 +610,7 @@ void OAuth::openBrowser()
 
     if (!isUrlValid(authorisationLink())) {
         qCWarning(lcOauth) << "URL validation failed";
-        QMetaObject::invokeMethod(qApp, "slotShowGuiMessage", Qt::QueuedConnection,
-            Q_ARG(QString, tr("Oauth2 Error")),
-            Q_ARG(QString, tr("Oauth2 authentication requires a secured connection.")));
-        emit result(Error, QString());
+        Q_EMIT result(ErrorInsecureUrl, QString());
         return;
     }
 
@@ -674,12 +703,12 @@ void AccountBasedOAuth::refreshAuthentication(const QString &refreshToken)
                 QString accessToken;
                 QString newRefreshToken = refreshToken;
                 // https://developer.okta.com/docs/reference/api/oidc/#response-properties-2
-                const QString error = data.value(QStringLiteral("error")).toString();
-                if (!error.isEmpty()) {
-                    if (error == QLatin1String("invalid_grant") || error == QLatin1String("invalid_request")) {
+                const QString errorString = data.value(QStringLiteral("error")).toString();
+                if (!errorString.isEmpty()) {
+                    if (errorString == QLatin1String("invalid_grant") || errorString == QLatin1String("invalid_request")) {
                         newRefreshToken.clear();
                     } else {
-                        qCWarning(lcOauth) << "Error while refreshing the token:" << error << data.value(QStringLiteral("error_description")).toString();
+                        qCWarning(lcOauth) << "Error while refreshing the token:" << errorString << data.value(QStringLiteral("error_description")).toString();
                     }
                 } else if (reply->error() != QNetworkReply::NoError) {
                     qCWarning(lcOauth) << "Error while refreshing the token:" << reply->error() << ":" << reply->errorString() << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
