@@ -53,11 +53,10 @@ class OWNCLOUDSYNC_EXPORT OAuth : public QObject
 {
     Q_OBJECT
 public:
-    enum Result { NotSupported,
-        LoggedIn,
-        LoggedInAppPassword,
-        Error };
-    Q_ENUM(Result);
+    enum Result { NotSupported, LoggedIn, Error, ErrorInsecureUrl };
+    Q_ENUM(Result)
+    enum class TokenEndpointAuthMethods { client_secret_basic, client_secret_post };
+    Q_ENUM(TokenEndpointAuthMethods)
 
     OAuth(const QUrl &serverUrl, const QString &davUser, QNetworkAccessManager *networkAccessManager, const QVariantMap &dynamicRegistrationData, QObject *parent);
     ~OAuth() override;
@@ -66,6 +65,9 @@ public:
     void openBrowser();
     QUrl authorisationLink() const;
 
+    void setDavUser(const QString &davUser);
+    QString davUser() const;
+
     static void saveDynamicRegistrationDataForAccount(const AccountPtr &accountPtr, const QVariantMap &dynamicRegistrationData);
 
 Q_SIGNALS:
@@ -73,7 +75,7 @@ Q_SIGNALS:
      * The state has changed.
      * when logged in, token has the value of the token.
      */
-    void result(OAuth::Result result, const QString &userName = QString(), const QString &token = QString(), const QString &refreshToken = QString(), const QString &displayName = QString());
+    void result(OAuth::Result result, const QString &token = QString(), const QString &refreshToken = QString());
 
     /**
      * emitted when the call to the well-known endpoint is finished
@@ -98,11 +100,11 @@ protected:
 
     virtual void fetchWellKnown();
 
-    QNetworkReply *postTokenRequest(const QList<QPair<QString, QString>> &queryItems);
+    QNetworkReply *postTokenRequest(QUrlQuery &&queryItems);
 
 
 private:
-    void finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QString &userName, const QString &displayName, const QUrl &messageUrl);
+    void finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QUrl &messageUrl);
 
     QByteArray generateRandomString(size_t size) const;
 
@@ -114,6 +116,8 @@ private:
     QString _redirectUrl;
     QByteArray _pkceCodeVerifier;
     QByteArray _state;
+
+    TokenEndpointAuthMethods _endpointAuthMethod = TokenEndpointAuthMethods::client_secret_basic;
 };
 
 /**
