@@ -19,6 +19,7 @@
 #include "creds/httpcredentials.h"
 #include "theme.h"
 
+#include <QApplication>
 #include <QLoggingCategory>
 
 Q_LOGGING_CATEGORY(lcDetermineAuthTypeJob, "sync.networkjob.determineauthtype2", QtInfoMsg);
@@ -42,23 +43,24 @@ CoreJob *DetermineAuthTypeJobFactory::startJob(const QUrl &url, QObject *parent)
 
     auto job = new CoreJob(nam()->sendCustomRequest(req, "PROPFIND"), parent);
 
-    connect(job->reply(), &QNetworkReply::finished, job, [job] {
+    QObject::connect(job->reply(), &QNetworkReply::finished, job, [job] {
         switch (job->reply()->error()) {
         case QNetworkReply::AuthenticationRequiredError:
             break;
         case QNetworkReply::NoError:
-            setJobError(job, tr("Server did not ask for authorization"));
+            setJobError(job, QApplication::translate("DetermineAuthTypeJobFactory", "Server did not ask for authorization"));
             return;
         default:
-            setJobError(job, tr("Failed to determine auth type: %1").arg(job->reply()->errorString()));
+            setJobError(job, QApplication::translate("DetermineAuthTypeJobFactory", "Failed to determine auth type: %1").arg(job->reply()->errorString()));
             return;
         }
 
-        const auto authChallenge = job->reply()->rawHeader(QByteArrayLiteral("WWW-Authenticate")).toLower();
+        const auto reply = job->reply();
+        const auto authChallenge = reply->rawHeader(QByteArrayLiteral("WWW-Authenticate")).toLower();
 
-        const AuthType authType = [authChallenge, job]() {
+        const AuthType authType = [reply, authChallenge]() {
             // we fall back to basic in any case
-            if (authChallenge.contains(QByteArrayLiteral("bearer ")) || job->reply()->hasRawHeader(QByteArrayLiteral("X-Oc-App-Password"))) {
+            if (authChallenge.contains(QByteArrayLiteral("bearer ")) || reply->hasRawHeader(QByteArrayLiteral("X-Oc-App-Password"))) {
                 return AuthType::OAuth;
             } else {
                 if (authChallenge.isEmpty()) {
@@ -70,7 +72,7 @@ CoreJob *DetermineAuthTypeJobFactory::startJob(const QUrl &url, QObject *parent)
         }();
 
         qCInfo(lcDetermineAuthTypeJob) << "Auth type for" << job->reply()->url() << "is" << authType;
-        setJobResult(job, qVariantFromValue(authType));
+        setJobResult(job, QVariant::fromValue(authType));
     });
 
     return job;
