@@ -266,7 +266,7 @@ void PropfindJob::start()
     QByteArray data;
     {
         QTextStream stream(&data, QIODevice::WriteOnly);
-        stream.setCodec("UTF-8");
+        stream.setEncoding(QStringConverter::Utf8);
         stream << QByteArrayLiteral("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                                     "<d:propfind xmlns:d=\"DAV:\">"
                                     "<d:prop>");
@@ -346,6 +346,7 @@ const QHash<QString, qint64> &PropfindJob::sizes() const
 AvatarJob::AvatarJob(AccountPtr account, const QString &userId, int size, QObject *parent)
     : AbstractNetworkJob(account, account->url(), QStringLiteral("remote.php/dav/avatars/%1/%2.png").arg(userId, QString::number(size)), parent)
 {
+    setStoreInCache(true);
 }
 
 void AvatarJob::start()
