@@ -12,28 +12,31 @@
  * for more details.
  */
 
-#include "config.h"
 #include "propagateupload.h"
-#include "owncloudpropagator_p.h"
-#include "networkjobs.h"
 #include "account.h"
+#include "filesystem.h"
+#include "networkjobs.h"
+#include "owncloudpropagator_p.h"
+#include "propagateremotedelete.h"
+#include "propagatorjobs.h"
+#include "syncengine.h"
+
+#include "common/asserts.h"
+#include "common/checksums.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/utility.h"
-#include "filesystem.h"
-#include "propagatorjobs.h"
-#include "common/checksums.h"
-#include "syncengine.h"
-#include "propagateremotedelete.h"
-#include "common/asserts.h"
 
-#include <QNetworkAccessManager>
-#include <QFileInfo>
+#include "libsync/theme.h"
+
 #include <QDir>
+#include <QFileInfo>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QNetworkAccessManager>
+
+#include <chrono>
 #include <cmath>
-#include <cstring>
 
 using namespace std::chrono_literals;
 
@@ -56,11 +59,11 @@ Q_LOGGING_CATEGORY(lcPropagateUploadNG, "sync.propagator.upload.ng", QtInfoMsg)
 static bool fileIsStillChanging(const SyncFileItem &item)
 {
     const QDateTime modtime = Utility::qDateTimeFromTime_t(item._modtime);
-    const qint64 msSinceMod = modtime.msecsTo(QDateTime::currentDateTimeUtc());
+    const auto secondsSinceMod = std::chrono::seconds(modtime.secsTo(QDateTime::currentDateTimeUtc()));
 
-    return std::chrono::milliseconds(msSinceMod) < SyncEngine::minimumFileAgeForUpload
+    return secondsSinceMod < SyncEngine::minimumFileAgeForUpload
         // if the mtime is too much in the future we *do* upload the file
-        && msSinceMod > -10000;
+        && secondsSinceMod > -1s;
 }
 
 PUTFileJob::PUTFileJob(AccountPtr account, const QUrl &url, const QString &path, std::unique_ptr<QIODevice> &&device, const QMap<QByteArray, QByteArray> &headers, int chunk, QObject *parent)
@@ -93,10 +96,9 @@ void PUTFileJob::finished()
 {
     _device->close();
 
-    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
-                     << replyStatusString()
+    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS" << replyStatusString()
                      << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt()
-                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
+                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
 }
 
 void PUTFileJob::newReplyHook(QNetworkReply *reply)
@@ -173,6 +175,13 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
         return;
     }
 
+    // we must be able to read the file
+    if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
+        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
+        return;
+    }
+
     // Compute the content checksum.
     auto computeChecksum = new ComputeChecksum(this);
     computeChecksum->setChecksumType(checksumType);
@@ -196,6 +205,14 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(CheckSums::Algor
         return;
     }
 
+    const QString filePath = propagator()->fullLocalPath(_item->_file);
+    // we must be able to read the file
+    if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
+        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
+        return;
+    }
+
     // Compute the transmission checksum.
     auto computeChecksum = new ComputeChecksum(this);
     if (uploadChecksumEnabled()) {
@@ -208,7 +225,6 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(CheckSums::Algor
         this, &PropagateUploadFileCommon::slotStartUpload);
     connect(computeChecksum, &ComputeChecksum::done,
         computeChecksum, &QObject::deleteLater);
-    const QString filePath = propagator()->fullLocalPath(_item->_file);
     computeChecksum->start(filePath);
 }
 
@@ -231,7 +247,7 @@ void PropagateUploadFileCommon::slotStartUpload(CheckSums::Algorithm transmissio
         done(SyncFileItem::SoftError, tr("File Removed"));
         return;
     }
-    _item->_size = FileSystem::getSize(fullFilePath);
+    _item->_size = FileSystem::getSize(QFileInfo{fullFilePath});
 
     const time_t prevModtime = _item->_modtime; // the _item value was set in PropagateUploadFile::start()
     // but a potential checksum calculation could have taken some time during which the file could
@@ -281,7 +297,7 @@ bool UploadDevice::open(QIODevice::OpenMode mode)
 
     // Get the file size now: _file.fileName() is no longer reliable
     // on all platforms after openAndSeekFileSharedRead().
-    auto fileDiskSize = FileSystem::getSize(_file.fileName());
+    auto fileDiskSize = FileSystem::getSize(QFileInfo{_file.fileName()});
 
     QString openError;
     if (!FileSystem::openAndSeekFileSharedRead(&_file, &openError, _start)) {
@@ -387,7 +403,7 @@ void UploadDevice::giveBandwidthQuota(qint64 bwq)
 {
     if (!atEnd()) {
         _bandwidthQuota = bwq;
-        QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection); // tell QNAM that we have quota
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection); // tell QNAM that we have quota
     }
 }
 
@@ -395,7 +411,7 @@ void UploadDevice::setBandwidthLimited(bool b)
 {
     if (_bandwidthLimited != b) {
         _bandwidthLimited = b;
-        QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection);
     }
 }
 
@@ -403,7 +419,7 @@ void UploadDevice::setChoked(bool b)
 {
     _choked = b;
     if (!_choked) {
-        QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection);
     }
 }
 
@@ -471,15 +487,15 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
         emit propagator()->insufficientRemoteStorage();
     } else if (_item->_httpErrorCode == 405) {
         status = SyncFileItem::FileIgnored;
-        _item->_instruction = CSYNC_INSTRUCTION_IGNORE;
+        _item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
         errorString = tr("Upload of file rejected by server");
     } else if (_item->_httpErrorCode == 491) {
         status = SyncFileItem::FileIgnored;
-        _item->_instruction = CSYNC_INSTRUCTION_IGNORE;
+        _item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
         errorString = tr("Invalid file path");
     } else if (_item->_httpErrorCode == 492) {
         status = SyncFileItem::FileIgnored;
-        _item->_instruction = CSYNC_INSTRUCTION_IGNORE;
+        _item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
         errorString = tr("File is sparse (contains large amounts of zeroes)");
     }
 
@@ -488,12 +504,13 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
 
 void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, qint64 fileSize)
 {
-    job->setTimeout(qBound<std::chrono::seconds>(
-        job->timeoutSec(),
-        // Calculate 3 minutes for each gigabyte of data
-        std::chrono::minutes(static_cast<quint64>((3min).count() * fileSize / 1e9)),
-        // Maximum of 30 minutes
-        30min));
+    // Calculate 3 minutes for each gigabyte of data
+    const auto timeout = std::chrono::minutes(static_cast<quint64>((3min).count() * fileSize / 1e9));
+    // Maximum of 30 minutes
+    // not less than the default/current val
+    if (timeout > job->timeoutSec() && timeout < 30min) {
+        job->setTimeout(timeout);
+    }
 }
 
 // This function is used whenever there is an error occuring and jobs might be in progress
@@ -522,7 +539,7 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
     headers[QByteArrayLiteral("Content-Type")] = QByteArrayLiteral("application/octet-stream");
     headers[QByteArrayLiteral("X-OC-Mtime")] = QByteArray::number(qint64(_item->_modtime));
 
-    if (_item->_file.contains(QLatin1String(".sys.admin#recall#"))) {
+    if (Q_UNLIKELY(Theme::instance()->enableCernBranding() && _item->_file.contains(QLatin1String(".sys.admin#recall#")))) {
         // This is a file recall triggered by the admin.  Note: the
         // recall list file created by the admin and downloaded by the
         // client (.sys.admin#recall#) also falls into this category
@@ -534,8 +551,7 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
     }
 
     if (!_item->_etag.isEmpty() && _item->_etag != QLatin1String("empty_etag")
-        && _item->_instruction != CSYNC_INSTRUCTION_NEW // On new files never send a If-Match
-        && _item->_instruction != CSYNC_INSTRUCTION_TYPE_CHANGE
+        && (_item->instruction() & ~(CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_TYPE_CHANGE)) // On new files never send a If-Match
         && !_deleteExisting) {
         // We add quotes because the owncloud server always adds quotes around the etag, and
         //  csync_owncloud.c's owncloud_file_id always strips the quotes.
@@ -583,28 +599,20 @@ void PropagateUploadFileCommon::finalize()
         return;
     }
 
-
-#ifdef Q_OS_WIN
-    m_fileLock.close();
-#endif
     // Update the database entry
     const auto result = propagator()->updateMetadata(*_item);
     if (!result) {
         done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
         return;
-    } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
-        done(SyncFileItem::SoftError, tr("The file %1 is currently in use").arg(_item->_file));
-        return;
     }
 
     // Files that were new on the remote shouldn't have online-only pin state
     // even if their parent folder is online-only.
-    if (_item->_instruction == CSYNC_INSTRUCTION_NEW
-        || _item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+    if (_item->instruction() & (CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_TYPE_CHANGE)) {
         auto &vfs = propagator()->syncOptions()._vfs;
         const auto pin = vfs->pinState(_item->_file);
         if (pin && *pin == PinState::OnlineOnly) {
-            vfs->setPinState(_item->_file, PinState::Unspecified);
+            std::ignore = vfs->setPinState(_item->_file, PinState::Unspecified);
         }
     }
 
@@ -656,7 +664,7 @@ void PropagateUploadFileCommon::abortNetworkJobs(
             // Connect to finished signal of job reply to asynchonously finish the abort
             connect(reply, &QNetworkReply::finished, this, oneAbortFinished);
         }
-        reply->abort();
+        job->abort();
     }
 
     if (*runningCount == 0 && abortType == AbortType::Asynchronous)
